import webbrowser

import nltk
import requests

nltk.download('popular')
from nltk.stem import WordNetLemmatizer

lemmatizer = WordNetLemmatizer()
import pickle
import numpy as np

from keras.models import load_model
import json
import random

model = load_model('model.h5')
intents = json.loads(open('data.json').read())
words = pickle.load(open('texts.pkl', 'rb'))
classes = pickle.load(open('labels.pkl', 'rb'))


def clean_up_sentence(sentence):
    # tokenize the pattern - split words into array
    sentence_words = nltk.word_tokenize(sentence)
    # stem each word - create short form for word
    sentence_words = [lemmatizer.lemmatize(word.lower()) for word in sentence_words]
    return sentence_words


# return bag of words array: 0 or 1 for each word in the bag that exists in the sentence

def bow(sentence, words, show_details=True):
    # tokenize the pattern
    sentence_words = clean_up_sentence(sentence)
    # bag of words - matrix of N words, vocabulary matrix
    bag = [0] * len(words)
    for s in sentence_words:
        for i, w in enumerate(words):
            if w == s:
                # assign 1 if current word is in the vocabulary position
                bag[i] = 1
                if show_details:
                    print("found in bag: %s" % w)
    return (np.array(bag))


def predict_class(sentence, model):
    # filter out predictions below a threshold
    p = bow(sentence, words, show_details=False)
    res = model.predict(np.array([p]))[0]
    ERROR_THRESHOLD = 0.25
    results = [[i, r] for i, r in enumerate(res) if r > ERROR_THRESHOLD]
    # sort by strength of probability
    results.sort(key=lambda x: x[1], reverse=True)
    return_list = []
    for r in results:
        return_list.append({"intent": classes[r[0]], "probability": str(r[1])})
    return return_list


def getResponse(ints, intents_json):
    tag = ints[0]['intent']
    list_of_intents = intents_json['intents']
    for i in list_of_intents:
        if (i['tag'] == tag):
            result = random.choice(i['responses'])
            break
    return result


def chatbot_response(msg):
    ints = predict_class(msg, model)
    res = getResponse(ints, intents)
    return res


from flask import Flask, render_template, request
from flask import Flask, request, jsonify
import joblib
import pandas as pd

app = Flask(__name__)
app.static_folder = 'static'

# Load the saved model
loaded_model = joblib.load('rf_model_learning_style.pkl')


@app.route("/")
def home():
    return render_template("index.html")

@app.route("/learning_style")
def learningStyle():
    return render_template("learningStyle.html")

@app.route("/teaching_style")
def teachingStyle():
    return render_template("teachingStyle.html")

@app.route("/behavior")
def behavior():
    return render_template("behavior.html")


@app.route("/get")
def get_bot_response():
    userText = request.args.get('msg')
    return chatbot_response(userText)



@app.route('/predict_learning_style', methods=['POST'])
def predict_learning_style():
    try:
        # Get the input data from the POST request
        data = request.json
        input_list = data['input']

        # Prepare the input data for prediction
        input_df = pd.DataFrame([input_list])

        # Make predictions using the loaded model
        predictions = loaded_model.predict(input_df)

        # Define a label mapping
        label_mapping = {0: 'Auditory', 1: 'Kinesthetic', 2: 'Read/Write', 3: 'Read/Write', 4: 'Visual'}

        # Map the predictions to labels
        predicted_labels = [label_mapping[prediction] for prediction in predictions]

        print(predicted_labels)

        # Return the predicted labels as JSON response
        return jsonify({'predicted_labels': predicted_labels})

    except Exception as e:
        return jsonify({'error': str(e)}), 400

loaded_modelT = joblib.load('predicting_teaching_style.pkl')

@app.route('/predict_teaching_style', methods=['POST'])
def predict_teaching_style():
    try:
        # Get the input data from the POST request
        data = request.json
        input_list = data['input']

        # Prepare the input data for prediction
        input_df = pd.DataFrame([input_list])

        # Make predictions using the loaded model
        predictions = loaded_modelT.predict(input_df)

        # Define a label mapping
        label_mapping = {
            0: 'HANDS ON EXPERIENCE',
            1: 'INDIVIDUALIZED SUPPORT',
            2: 'MULTISENSORY INSTRUCTION',
            3: 'PLAY BASED LEARNING',
            4: 'WHOLE GROUP AND SMALL GROUP INSTRUCTION'
        }

        # Map the predictions to labels
        predicted_labels = [label_mapping[prediction] for prediction in predictions]

        # Return the predicted labels as JSON response
        return jsonify({'predicted_labels': predicted_labels})

    except Exception as e:
        return jsonify({'error': str(e)}), 400

loaded_modelB = joblib.load('behaviour_prediction.pkl')

@app.route('/predict_behavior', methods=['POST'])
def predict_behavior():
    try:
        # Get the input data from the POST request
        data = request.json
        input_list = data['input']

        # Prepare the input data for prediction
        input_df = pd.DataFrame([input_list])

        # Make predictions using the loaded model
        predictions = loaded_modelB.predict(input_df)

        # Define a label mapping
        label_mapping = {
            0: 'Aggressive',
            1: 'Hyperactive',
            2: 'Oppositional',
            3: 'Tantrum',
            4: 'Withdrawal'
        }

        # Map the predictions to labels
        predicted_labels = [label_mapping[prediction] for prediction in predictions]

        # Return the predicted labels as JSON response
        return jsonify({'predicted_labels': predicted_labels})

    except Exception as e:
        return jsonify({'error': str(e)}), 400



if __name__ == "__main__":
    app.run()
